<?
/* --------------------------------------------------------------------
   Copyright (c) 2008-2012 Daniel Pedemont (dan@pedemont.net)
   -------------------------------------------------------------------- */
/**
 * utility library
 *
 * misc. functions for CLI, utility, curl, http, etc...
 *
 * @link http://dev.pedemont.com/sonic/
 * @link http://dev.pedemont.com/sonic/downloads/src/lib/sonic.inc
 * @package sonic
 * @version 0.0.1
 * @abstract secondary utility include library
 */

define('SONIC_UTIL_VERSION', '0.1.1');

/**
 * HTTP request (via cURL)
 *
 * NB, HTTP header request must contain the 'Host' tag as the destination
 *
 * @param string $http_request HTTP request to be sent
 * @param array $curl_opt option flags, (string) tmp, (string) default_scheme, (boolean) tmp_dump_out, (boolean) return_header
 * @return mixed see curl_exec()
*/ 
function sonic_http_request($http_request, $curl_opt = NULL)
{
  if (gettype($curl_opt) == 'array') {}
  else if ($curl_opt === NULL)
  {
    $curl_opt = array();
  }
  else
    return false;

  if ($curl_opt['tmp'] === NULL)
    $curl_opt['tmp'] = SONIC_TMP_PATH;

  if ($curl_opt['default_scheme'] === NULL)
    $curl_opt['default_scheme'] = 'http';

  if ($curl_opt['tmp_dump_out'] === NULL)
    $curl_opt['tmp_dump_out'] = false;

  if ($curl_opt['header'] === NULL)
    $curl_opt['header'] = false;

  if ($curl_opt['follow_location'] === NULL)
    $curl_opt['follow_location'] = true;

  if ($curl_opt['cookie_session'] === NULL)
    $curl_opt['cookie_session'] = false;

  if ($curl_opt['binary_transfer'] === NULL)
    $curl_opt['binary_transfer'] = true;

  if ($curl_opt['tmp'] && is_writable($curl_opt['tmp']))
  {
    if ($curl_opt['cookie_file'] === NULL OR $curl_opt['cookie_file'] === true)
      $curl_opt['cookie_file'] = $curl_opt['tmp'].DIRECTORY_SEPARATOR.'cookie_jar.txt';

    if ($curl_opt['cookie_jar'] === NULL OR $curl_opt['cookie_jar'] === true)
      $curl_opt['cookie_jar'] = $curl_opt['tmp'].DIRECTORY_SEPARATOR.'cookie_jar.txt';
  }
  else
  {
    unset($curl_opt['tmp']);
    $curl_opt['tmp_dump_out'] = false;

    if ($curl_opt['cookie_file'] === true)
      unset($curl_opt['cookie_file']);

    if ($curl_opt['cookie_jar'] === true)
      unset($curl_opt['cookie_jar']);
  }

  if (isset($curl_opt['cookie_file']))                                                                       {
    if (is_file($curl_opt['cookie_file']) && is_writable($curl_opt['cookie_file'])) {}
    else if (@touch($curl_opt['cookie_file'])) {}
    else
      unset($curl_opt['cookie_file']);
  }
  else
    unset($curl_opt['cookie_file']);

  if (isset($curl_opt['cookie_jar']))
  {
    if (is_file($curl_opt['cookie_jar']) && is_writable($curl_opt['cookie_jar'])) {}
    else if (@touch($curl_opt['cookie_jar'])) {}
    else
      unset($curl_opt['cookie_jar']);
  }
  else
    unset($curl_opt['cookie_jar']);

//  if ($curl_opt[''] === NULL)
//    $curl_opt[''] = ;

  //|CONNECT?
  $matches = NULL;
  if (preg_match('/^((?:GET|POST))\s+(.*)\s+HTTP\/(1\.[01])\s*(\r{0,1})$/Uim', $http_request, $matches));
  else
    return false;

  $hhr_method = $matches[1];
  $hhr_url = $matches[2];
  $hhr_ver = $matches[3];
  $hhr_eol = $matches[4]."\n";

  //split request header into array of vars for curl
  $hhr_split_p = '/\r{0,1}\n\r{0,1}\n/';

  //header + get/post AND post data
  list($hhr_header, $hhr_body) = preg_split($hhr_split_p, $http_request, 2);

  //header in array (- get/post)
  $hhr_header_a = sonic_header2array($hhr_header);

  if ($hhr_method == 'POST' && $c_post_data = sonic_http_request_post($http_request, $c_post_FILES, $curl_opt['tmp']))
  {
    foreach ($c_post_FILES as $c_post_fn => $c_post_f_data)
    {
      if (($c_post_f_data['error'] + 0) == 0)
      {
	$c_post_data[0][] = $c_post_fn;
	$c_post_data[1][] = '@'.$c_post_f_data['tmp_name'];
      }
    }
    unset($c_post_FILES);
  }

  //make curl url
  if (preg_match('/^.*:\/\//', $hhr_url))
  {
    $c_url = $hhr_url;
  }
  else if ($curl_opt['default_host'])
  {
    $c_url = $curl_opt['default_scheme'].'://'
		.$curl_opt['default_host']
		.($curl_opt['default_port'] ? ':'.$curl_opt['default_port'] : '')
		.$hhr_url;

    //NB, curl wont send empty Host:, will either omit or overwrite, so best to allow internal overwrite
    if ($hhr_header_a['Host']);
    else
      unset($hhr_header_a['Host']);
  }
  else if ($hhr_header_a['Host'])
  {
    $c_url = $curl_opt['default_scheme'].'://'.$hhr_header_a['Host'].$hhr_url;
  }
//  else if (preg_match('/^\/\//', $hhr_url))
//  {
//    return false;
//  }
  else
    return false;

//make hhr header array into curl header array
//NB, $header = array('PART: VALUE', 'TAG: VALUE');
  $_c_hhr_header_a = $hhr_header_a;

  unset($_c_hhr_header_a['Proxy-Connection']);
//  unset($_c_hhr_header_a['Host']);

/*
  //if this is sent in httpheader, and not via curl_setopt
  //results will not be sent back via decoder filter
  //ie. send gzip in httpheader, return data is gzipped
  if ($_c_hhr_header_a['Accept-Encoding'])
  {
    //we setopt this below, seems safe to send also
//    unset($_c_hhr_header_a['Accept-Encoding']);
  }
*/

  //nb,
  // curl needs url
  // url has standard :80 will NOT work, TODO: filter out, maybe bug with environment?
  // url non standard port eg. :82 will work
  // httpheaders, host needs removing & accept appends (bug in current curl)

  $c_h = curl_init();

  if (is_string($curl_opt['interface']))
    curl_setopt($c_h, CURLOPT_INTERFACE, $curl_opt['interface']);

  curl_setopt($c_h, CURLOPT_URL, $c_url);
//  curl_setopt($c_h, CURLOPT_PORT, 80);

  //this wont be set if sent in httpheader
  if ($_c_hhr_header_a['Accept-Encoding'])
    curl_setopt($c_h, CURLOPT_ENCODING, $_c_hhr_header_a['Accept-Encoding']);

  if (is_string($curl_opt['cookie']))
    curl_setopt($c_h, CURLOPT_COOKIE, $curl_opt['cookie']);

  //resets previous sessions
  if (is_bool($curl_opt['cookie_session']))
    curl_setopt($c_h, CURLOPT_COOKIESESSION, $curl_opt['cookie_session']);

  //sets the cookie file to read/write _current_ session
  if (is_string($curl_opt['cookie_jar']))
    curl_setopt($c_h, CURLOPT_COOKIEJAR, $curl_opt['cookie_jar']);

  //sets the cookie file to read (previous sessions ie. same name)
  if (is_string($curl_opt['cookie_file']))
    curl_setopt($c_h, CURLOPT_COOKIEFILE, $curl_opt['cookie_file']);

  if (is_bool($curl_opt['verbose']))
    curl_setopt($c_h, CURLOPT_VERBOSE, $curl_opt['verbose']);

  if (is_numeric($curl_opt['timeout']))
    curl_setopt($c_h, CURLOPT_TIMEOUT, $curl_opt['timeout']);

  if (is_bool($curl_opt['no_progress']))
    curl_setopt($c_h, CURLOPT_NOPROGRESS, $curl_opt['no_progress']);

  if (is_numeric($curl_opt['max_connects']))
    curl_setopt($c_h, CURLOPT_MAXCONNECTS, $curl_opt['max_connects']);

//php DNE 
//curl_setopt($c_h, CURLOPT_TCP_NODELAY, true);
//curl_setopt($c_h, CURLOPT_IGNORE_CONTENT_LENGTH, true);
//curl_setopt($c_h, CURLOPT_POSTREDIR, 1);

  if (is_bool($curl_opt['fresh_connect']))
    curl_setopt($c_h, CURLOPT_FRESH_CONNECT, $curl_opt['fresh_connect']);

  if (is_bool($curl_opt['forbid_reuse']))
    curl_setopt($c_h, CURLOPT_FORBID_RESUSE, $curl_opt['forbid_reuse']);

  if (is_bool($curl_opt['ssl_verify_peer']))
    curl_setopt($c_h, CURLOPT_SSL_VERIFYPEER, $curl_opt['ssl_verify_peer']);

  if (is_numeric($curl_opt['ssl_verify_host']))
    curl_setopt($c_h, CURLOPT_SSL_VERIFYHOST, $curl_opt['ssl_verify_host']);

  if (is_numeric($curl_opt['ssl_version']))
    curl_setopt($c_h, CURLOPT_SSLVERSION, $curl_opt['ssl_version']);

  if (is_bool($curl_opt['binary_transfer']))
    curl_setopt($c_h, CURLOPT_BINARYTRANSFER, $curl_opt['binary_transfer']);

  if ($curl_opt['follow_location'])
  {
    if ($curl_opt['follow_location'] === 'manual');
    else
    {
      curl_setopt($c_h, CURLOPT_FOLLOWLOCATION, true);

      if (is_numeric($curl_opt['max_redirs']))
	curl_setopt($c_h, CURLOPT_MAXREDIRS, $curl_opt['max_redirs']);
    }
  }

  if ($curl_opt['follow_location'] === 'manual')
  {
    curl_setopt($c_h, CURLOPT_HEADER, 1);
  }
  else if (is_bool($curl_opt['header']))
  {
    curl_setopt($c_h, CURLOPT_HEADER, $curl_opt['header']);
  }

  curl_setopt($c_h, CURLOPT_RETURNTRANSFER, true);

  if ($hhr_method == 'POST' && $c_post_data)
  {
    curl_setopt($c_h, CURLOPT_POST, true); //needed?
    //NB, if sent as array should include blank 'expect:' incase 100 continue is not returned
    //passing an array to curlopt_postfields will encode the data as multipart/form-data, while passing a url-encoded string will encode the data as application/x-www-form-urlencoded. 

    //Content-Type: multipart/form-data; boundary=delim
    if (preg_match('/\s*multipart\s*[^\s]\s*form-data;\s+boundary\s*=([^\r\n]+)$/i', $_c_hhr_header_a['Content-Type']))
    {
      if ($_c_hhr_header_a['Content-Length'])
        unset($_c_hhr_header_a['Content-Length']);

      //NB, identical keys will be clobbered, nature of PHP and CURL
      $c_post_data_fields = array();
      foreach ($c_post_data[1] as $_pi => $_pv)
      {
        $_pk = $c_post_data[0][$_pi];
	$c_post_data_fields[$_pk] = $_pv;
      }

      curl_setopt($c_h, CURLOPT_POSTFIELDS, $c_post_data_fields);
    }
    //Content-Type: application/x-www-form-urlencoded (default no check)
    else if (preg_match('/\s*application\s*[^\s]\s*x-www-form-/i', $_c_hhr_header_a['Content-Type']))
    {
      $c_post_data_s = '';

      foreach ($c_post_data[1] as $_pi => $_pv)
      {
        $_pk = $c_post_data[0][$_pi];
	$c_post_data_s .= '&'.urlencode($_pk).'='.urlencode($_pv);
      }

      if ($c_post_data_s)
	$c_post_data_s = substr($c_post_data_s, 1);

      curl_setopt($c_h, CURLOPT_POSTFIELDS, $c_post_data_s);
    }

//NB, will be overwritten either way
//    if ($_c_hhr_header_a['Content-Type'])
//      unset($_c_hhr_header_a['Content-Type']);
  }

  $c_hhr_header_a = array();
  foreach ($_c_hhr_header_a as $_chha_k => $_chha_v)
    array_push($c_hhr_header_a, $_chha_k.': '.$_chha_v);
  unset($_c_hhr_header_a);

  curl_setopt($c_h, CURLOPT_HTTPHEADER, $c_hhr_header_a);

  if (($out = curl_exec($c_h)) === false)
  {
//    var_dump(curl_error($c_h));

    return false;
  }

  curl_close($c_h);

  if ($curl_opt['tmp_dump_out'])
  {
    $c_file = rawurlencode($c_url);
//    $c_file = urlencode($c_url);

    if ($fp = fopen($curl_opt['tmp'].DIRECTORY_SEPARATOR.$c_file, 'wb'))
    {
      fputs($fp, $out);
      fclose($fp);
    }
  }

  if ($curl_opt['follow_location'] === 'manual')
  {
    if (is_numeric($curl_opt['max_redirs']));
    else
      $curl_opt['max_redirs'] = 5;

    $matches = NULL;
    if (preg_match('/^HTTP\/1\.[01]\s+30[12]\s*[^\r\n]*(\r{0,1})$/m', $out, $matches) && $curl_opt['max_redirs'] > 0)
    {
      $hhrsp_eol = $matches[1]."\n";
/*
      $out = sonic_http_header_split($out_all, $out_hresp_headers);

      foreach ($out_hresp_headers as $_ohh)
      {
	$_ohh_a = sonic_header2array($_ohh);

	if ($_ohh_a['Location'] === NULL);
	else
	  $_ohh_last_location = $_ohh_a['Location'];
      }
*/
      $hhrsp_header_100 = $matches = NULL;
      if (preg_match('/^(HTTP\/1\.[01]\s+100\s+Continue\s*\r{0,1}\n(?:.*\r{0,1}\n|))\r{0,1}\n/U', $out, $matches))
      {
	$hhrsp_header_100 = $matches[1];

	$out = preg_replace('/^HTTP\/1\.[01]\s+100\s+Continue\s*\r{0,1}\n(?:.*\r{0,1}\n|)\r{0,1}\n/U', '', $out);
      }

      $hhrsp_split_p = '/\r{0,1}\n\r{0,1}\n/';

      list($hhrsp_header, $hhrsp_body) = preg_split($hhrsp_split_p, $out, 2);

      $hhrsp_header_a = sonic_header2array($hhrsp_header);

      if ($hhrsp_header_a['Location']);
      else
	return false;

      $_hhr_header_a = $hhr_header_a;

      unset($_hhr_header_a['Content-Type']);
      unset($_hhr_header_a['Content-Length']);

      if ($_hhr_header_a['Referer']);
//NB, referer should be as the original, not sure if should set from 302 otherwise
      else
	$_hhr_header_a['Referer'] = $c_url;

      $_pu = parse_url($hhrsp_header_a['Location']);

      if ($_pu['host'])
	$_hhr_header_a['Host'] = $_pu['host'];

//  $hhr_method
//curl_setopt($c_h, CURLOPT_POSTREDIR, 1);

      $_http_request = 'GET '.$hhrsp_header_a['Location'].' HTTP/'.$hhr_ver.$hhr_eol;

      foreach ($_hhr_header_a as $_hrk => $_hrv)
	$_http_request .= $_hrk.': '.$_hrv.$hhr_eol;

      if ($curl_opt['cookie_session'] === 'strict');
      else
	$curl_opt['cookie_session'] = false;

      $curl_opt['max_redirs']--;

      $out = ($hhrsp_header_100 ? $hhrsp_header_100.$hhrsp_eol : '')
		.$hhrsp_header.$hhrsp_eol
		.$hhrsp_eol
		.sonic_http_request($_http_request, $curl_opt);
    }

    if ($curl_opt['header']);
    else
      $out = sonic_http_header_split($out, $hresp_headers, false);
  }

  return $out;
}

/**
 * split HTTP header from body
 *
 * NB, this function is untested, mainly for sonic_http_request() as CURLOPT_HEADER=1
 *
 * this function will _not_ return the split of the _first_ HTTP header from its body.
 * WARN, you do not want use this function to prevent HTTP smuggling.
 *
 * this function will take the last header as the real, final and only header associated
 * with the body. usually you should split the first header and the body will contain
 * any other remaining/smuggled headers (and not be treated as such). however when
 * processing a response from cURL with headers returned, all headers cURL encountered to
 * reach the final header will be returned (and you will probably want to filter them out).
 *
 * this function will return the split of the _last_ HTTP header from its body.
 *
 * @param string $hresp HTTP response
 * @param array &$hresp_headers HTTP response header array
 * @param array $opts option flags, header_max
 * @return array array with index pair of header and body
*/ 
function sonic_http_header_split($hresp, &$hresp_headers, $header_max = false)
{
  $header_count_skip = false;

  if ($header_max === NULL)
  {
    $header_max = 5;
  }
  else if ($header_max === false)
  {
    $header_max = 0;
    $header_count_skip = true;
  }
  else
  {
    $header_max = (int) $header_max + 0;

    if ($header_max === 0)
    {
      $header_count_skip = true;
    }
    else if ($header_max > 0) {}
    else
      $header_max = 5;
  }

  $hresp_headers = array();
  $header_i = 0;
  $o_body = $_o_body = $hresp;

  $matches = NULL;
  while (preg_match('/^(HTTP\/.*)\r{0,1}\n\r{0,1}\n/sU', $_o_body, $matches))
  {
    if ($header_count_skip) {}
    else if ($header_i < $header_max) {}
    else
    {
      break;
    }

    array_push($hresp_headers, $matches[1]);

    $o_body = preg_replace('/^HTTP.*\r{0,1}\n\r{0,1}\n/sU', '', $_o_body, 1);

    if ($_o_body == $o_body)
      break;

    $_o_body = $o_body;

    $header_i++;
  }
  unset($_o_body, $header_i, $header_max);

  return $o_body;
}

function sonic_http_request_create(&$opts, &$hrf_post_data)
{
  if ($opts['carriage_return'] === NULL)
    $opts['carriage_return'] = true;

  if ($opts['carriage_return'])
    $_eol = "\r\n";
  else
    $_eol = "\n";

  if ($opts['emulate_browser'] === NULL OR $opts['emulate_browser'] === true)
    $opts['emulate_browser'] = 'random';

  $hr = '';
  $hrf_post_data = NULL;

  $opts['method'] = strtoupper($opts['method']);

  switch ($opts['method'])
  {
    case 'GET':
    case 'POST':
    break;

    default:
      $opts['method'] = 'GET';
  }

  if ($opts['URL'] === NULL)
  {
    $hrf_host = '';
    $hrf_uri = '/';
  }
  else
  {
    $_pu = parse_url($opts['URL']);

    //NB, may have fragmented on #[0-9]+; encoding, fragments shouldnt be included in the request anyway
    if ($_pu['fragment'] == '');
    else
    {
      $_pu['query'] .= '#'.$_pu['fragment'];
      unset($_pu['fragment']);
    }

    $hrf_host = $_pu['host'];

    if (is_numeric($_pu['port']))
    {
      switch ($_pu['scheme'])
      {
	case 'http':
	  if ($_pu['port'] == '80');
	  else
	    $hrf_host .= ':'.$_pu['port'];
	break;

	case 'https':
	  if ($_pu['port'] == '443');
	  else
	    $hrf_host .= ':'.$_pu['port'];
	break;
      }
    }

    if ($opts['method_query'] === NULL)
      $opts['method_query'] = $opts['method'];
    else
      $opts['method_query'] = strtoupper($opts['method_query']);

    switch ($opts['method_query'])
    {
      case 'GET':
	$hrf_uri = $_pu['path'].($_pu['query'] ? '?'.$_pu['query'] : '');
      break;

      case 'POST':
	$hrf_uri = $_pu['path'];

	sonic_parse_str($_pu['query'], $_ps);

	$_pu['query_encoded'] = '';
	foreach ($_ps as $k => $v)
	{
	  $_pu['query_encoded'] .= '&'.urlencode($k).'='.urlencode($v);
	}

	if ($_pu['query_encoded'])
	  $_pu['query_encoded'] = substr($_pu['query_encoded'], 1);

	$hrf_post_data = &$_pu['query_encoded'];
      break;
    }
  }

//NB, http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.23

  $hr = $opts['method'].' '.$hrf_uri.' HTTP/1.1'.$_eol;

  if ($opts['emulate_browser']);
  else
  {
    $hr .= 'Host: '.$hrf_host.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	 .'Content-Length: '.strlen($hrf_post_data).$_eol
	:
	  ''
	);

    return $hr;
  }
  
  $browsers = array(
		'linux_ubuntu_firefox_8',
		'linux_ubuntu_chrome_15.0.874.106',
//
		'win_xp_firefox_3.5.5',
		'win_xp_firefox_3.5.6',
		'win_xp_firefox_3.6.3',
		'win_xp_chrome_14.0.835.186',
		'win_xp_chrome_15.0.874.121',
		'win_xp_ie_6',
		'win_xp_ie_7',
		'win_xp_ie_8',
	);

  if (in_array($opts['emulate_browser'], $browsers, true));
  else
  {
    switch ($opts['emulate_browser'])
    {
      case 'linux_ubuntu_firefox':
	$opts['emulate_browser'] = 'linux_ubuntu_firefox_8';
      break;

      case 'linux_ubuntu_chrome':
	$opts['emulate_browser'] = 'linux_ubuntu_chrome_15.0.874.106';
      break;

      case 'win_xp_firefox':
	$opts['emulate_browser'] = 'win_xp_firefox_3.6.3';
      break;

      case 'win_xp_chrome':
	$opts['emulate_browser'] = 'win_xp_chrome_15.0.874.121';
      break;

      case 'win_xp_ie':
	$opts['emulate_browser'] = 'win_xp_ie_8';
      break;

      default:
//	$opts['emulate_browser'] = $browsers[array_rand($browsers, 1)];
//
//http://www.w3schools.com/browsers/browsers_os.asp
//http://www.w3schools.com/browsers/browsers_stats.asp

	$_r = rand(1, 100);

	if ($_r > 5)			//95% win (2,3,4,5,6,7,8,9)
	{
	  $_r = rand(1, 100);

	  if ($_r > 20 && $_r < 60)
	  {
	    $_bi = rand(2, 4);
	  }
	  else if ($_r > 20)
	  {
	    $_bi = rand(5, 6);
	  }
	  else
	  {
	    if ($_r > 2 && $_r < 5)
	    {
	      $_bi = 8;
	    }
	    else if ($_r > 2)
	    {
	      $_bi = 9;
	    }
	    else
	      $_bi = 7;
	  }
	}
	else					//5% linux (0,1)
	{
	  $_bi = rand(0, 1);
	}

	$opts['emulate_browser'] = $browsers[$_bi];
    }
  }

  switch ($opts['emulate_browser'])
  {
    case 'linux_ubuntu_firefox_8':
      $hr .= 'Host: '.$hrf_host.$_eol
	.'User-Agent: Mozilla/5.0 (Ubuntu; X11; Linux x86_64; rv:8.0) Gecko/20100101 Firefox/8.0'.$_eol
	.'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'.$_eol
	.'Accept-Language: en-us,en;q=0.5'.$_eol
	.'Accept-Encoding: gzip, deflate'.$_eol
	.'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7'.$_eol
	.'Connection: keep-alive'.$_eol
	.($opts['referer'] ? 'Referer: '.$opts['referer'].$_eol : '')
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '')
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	 .'Content-Length: '.strlen($hrf_post_data).$_eol
	:
	  ''
	);
    break;

    case 'linux_ubuntu_chrome_15.0.874.106':
      $hr .= 'Host: '.$hrf_host.$_eol
	.'Connection: keep-alive'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Length: '.strlen($hrf_post_data).$_eol
	 .'Cache-Control: max-age=0'.$_eol
	 .($opts['origin'] ?
	   'Origin: '.$opts['origin'].$_eol
	 :
	   (($hrf_host && $_pu['scheme']) ? 'Origin: '.$_pu['scheme'].'://'.$hrf_host.$_eol : '')
	 )
	:
	  ''
	)
	.'User-Agent: Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/11.10 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	:
	  ''
	)
	.'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'.$_eol
	.($opts['referer'] ? 'Referer: '.$opts['referer'].$_eol : '')
	.'Accept-Encoding: gzip,deflate,sdch'.$_eol
	.'Accept-Language: en-US,en;q=0.8'.$_eol
	.'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.3'.$_eol
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '');
    break;
//
    case 'win_xp_firefox_3.5.5':
      $hr .= 'Host: '.$hrf_host.$_eol
	.'User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.5) Gecko/20091102 Firefox/3.5.5'.$_eol
	.'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'.$_eol
	.'Accept-Language: en-us,en;q=0.5'.$_eol
	.'Accept-Encoding: gzip,deflate'.$_eol
	.'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7'.$_eol
	.'Keep-Alive: 300'.$_eol
	.'Connection: keep-alive'.$_eol
	.($opts['referer'] ? 'Referer: '.$opts['referer'].$_eol : '')
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '')
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	 .'Content-Length: '.strlen($hrf_post_data).$_eol
	:
	  ''
	);
    break;

    case 'win_xp_firefox_3.5.6':
      $hr .= 'Host: '.$hrf_host.$_eol
	.'User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.1.6) Gecko/20091201 Firefox/3.5.6'.$_eol
	.'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'.$_eol
	.'Accept-Language: en-us,en;q=0.5'.$_eol
	.'Accept-Encoding: gzip,deflate'.$_eol
	.'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7'.$_eol
	.'Keep-Alive: 300'.$_eol
	.'Connection: keep-alive'.$_eol
	.($opts['referer'] ? 'Referer: '.$opts['referer'].$_eol : '')
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '')
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	 .'Content-Length: '.strlen($hrf_post_data).$_eol
	:
	  ''
	);
    break;

    case 'win_xp_firefox_3.6.3':
      $hr .= 'Host: '.$hrf_host.$_eol
	.'User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.2.3) Gecko/20100401 Firefox/3.6.3'.$_eol
	.'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'.$_eol
	.'Accept-Language: en-us,en;q=0.5'.$_eol
	.'Accept-Encoding: gzip,deflate'.$_eol
	.'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7'.$_eol
	.'Keep-Alive: 115'.$_eol
	.'Connection: keep-alive'.$_eol
	.($opts['referer'] ? 'Referer: '.$opts['referer'].$_eol : '')
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '')
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	 .'Content-Length: '.strlen($hrf_post_data).$_eol
	:
	  ''
	);
    break;

    case 'win_xp_chrome_14.0.835.186':
      $hr .= 'Host: '.$hrf_host.$_eol
	.'Connection: keep-alive'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Length: '.strlen($hrf_post_data).$_eol
	 .'Cache-Control: max-age=0'.$_eol
	 .($opts['origin'] ?
	   'Origin: '.$opts['origin'].$_eol
	 :
	   (($hrf_host && $_pu['scheme']) ? 'Origin: '.$_pu['scheme'].'://'.$hrf_host.$_eol : '')
	 )
	:
	  ''
	)
	.'User-Agent: Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.186 Safari/535.1'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	:
	  ''
	)
	.'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'.$_eol
	.($opts['referer'] ?
	  'Referer: '.$opts['referer'].$_eol
	:
	  ''
	)
	.'Accept-Encoding: gzip,deflate,sdch'.$_eol
	.'Accept-Language: en-US,en;q=0.8'.$_eol
	.'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.3'.$_eol
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '');
    break;

    case 'win_xp_chrome_15.0.874.121':
      $hr .= 'Host: '.$hrf_host.$_eol
	.'Connection: keep-alive'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Length: '.strlen($hrf_post_data).$_eol
	 .'Cache-Control: max-age=0'.$_eol
	 .($opts['origin'] ?
	   'Origin: '.$opts['origin'].$_eol
	 :
	   (($hrf_host && $_pu['scheme']) ? 'Origin: '.$_pu['scheme'].'://'.$hrf_host.$_eol : '')
	 )
	:
	  ''
	)
	.'User-Agent: Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.121 Safari/535.2'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	:
	  ''
	)
	.'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8'.$_eol
	.($opts['referer'] ?
	  'Referer: '.$opts['referer'].$_eol
	:
	  ''
	)
	.'Accept-Encoding: gzip,deflate,sdch'.$_eol
	.'Accept-Language: en-US,en;q=0.8'.$_eol
	.'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.3'.$_eol
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '');
    break;

    case 'win_xp_ie_6':
      $hr .= 'Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/x-ms-application, application/x-ms-xbap, application/vnd.ms-xpsdocument, application/xaml+xml, */*'.$_eol
	.($opts['referer'] ? 'Referer: '.$opts['referer'].$_eol : '')
	.'Accept-Language: en-us'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	:
	  ''
	)
	.'Accept-Encoding: gzip, deflate'.$_eol
	.'User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)'.$_eol
	.'Host: '.$hrf_host.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Length: '.strlen($hrf_post_data).$_eol
	:
	  ''
	)
	.'Connection: Keep-Alive'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Cache-Control: no-cache'.$_eol
	:
	  ''
	)
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '');
    break;

    case 'win_xp_ie_7':
      $hr .= 'Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, */*'.$_eol
	.($opts['referer'] ? 'Referer: '.$opts['referer'].$_eol : '')
	.'Accept-Language: en-us'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	:
	  ''
	)
	.'UA-CPU: x86'.$_eol
	.'Accept-Encoding: gzip, deflate'.$_eol
	.'User-Agent: Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 2.0.50727)'.$_eol
	.'Host: '.$hrf_host.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Length: '.strlen($hrf_post_data).$_eol
	:
	  ''
	)
	.'Connection: Keep-Alive'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Cache-Control: no-cache'.$_eol
	:
	  ''
	)
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '');
    break;

    case 'win_xp_ie_8':
      $hr .= 'Accept: image/gif, image/jpeg, image/pjpeg, image/pjpeg, application/x-shockwave-flash, */*'.$_eol
	.($opts['referer'] ? 'Referer: '.$opts['referer'].$_eol : '')
	.'Accept-Language: en-us'.$_eol
	.'User-Agent: Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727)'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Type: application/x-www-form-urlencoded'.$_eol
	:
	  ''
	)
	.'Accept-Encoding: gzip, deflate'.$_eol
	.'Host: '.$hrf_host.$_eol
	.(($opts['method'] == 'POST') ?
	  'Content-Length: '.strlen($hrf_post_data).$_eol
	:
	  ''
	)
	.'Connection: Keep-Alive'.$_eol
	.(($opts['method'] == 'POST') ?
	  'Cache-Control: no-cache'.$_eol
	:
	  ''
	)
	.($opts['cookie'] ? 'Cookie: '.$opts['cookie'].$_eol : '');
    break;
  }

  return $hr;
}

?>
