<?
/* --------------------------------------------------------------------
   Copyright (c) 2008-2012 Daniel Pedemont (dan@pedemont.net)
   -------------------------------------------------------------------- */
/**
 * stream ping
 *
 * icmp echo ping-pong client
 *
 * @link http://dev.pedemont.com/sonic/
 * @link http://dev.pedemont.com/sonic/downloads/src/lib/streams/sonic.stream.ping.php
 * @package sonic_stream
 * @version 0.1.11
 * @abstract ping-pong
*/

define('SONIC_PING_VERSION', '0.1.13');

//NB, function names in rotate mode have no protection,
//    so dont use generic function names for clash safety

class SONIC_STREAM_ping extends SONIC_stream
{
  /**
   * time start
   *
   * @var integer
  */
  var $time_start = 0;
  /**
   * host
   *
   * @var string
  */
  var $host = '';
  /**
   * port number
   *
   * @var integer
  */
  var $port = 0;
  /**
   * time out limit in milliseconds
   *
   * @var integer
  */
  var $timeout_max = 300000; //deadline 300ms
  /**
   * packet data size
   *
   * eg. 56 bytes (7 * 8) + 8byte header = 64 bytes = 512 bits packet size
   *
   * @var integer
  */
  var $data_size = 7;
  /**
   * minimum response time in milliseconds
   *
   * @var integer
  */
  var $min = 0;
  /**
   * average response time in milliseconds
   *
   * @var integer
  */
  var $avg = 0;
  /**
   * largest/longest response time in milliseconds
   *
   * @var integer
  */
  var $max = 0;
  /**
   * total median deviation of response times in milliseconds
   *
   * @var integer
  */
  var $mdev_total = 0;
  /**
   * total response time in milliseconds
   *
   * @var integer
  */
  var $ms_total = 0;
  /**
   * sequence counter
   *
   * @var integer
  */
  var $icmp_seq = 0;

  /**
   * signal termination handler
   *
   * @param integer $sig posix process signal
  */
  function _sonic_stream_ping_handler($sig = NULL)
  {
//echo "_sonic_stream_ping_handler()\n";
    switch ($sig)
    {
      case SIGTERM:  //15
      case SIGQUIT:  // 3
      case SIGINT:   // 2
        $avg = $this->ms_total / $this->icmp_seq;
        $sdev = sqrt($this->mdev_total / $this->icmp_seq);
        if ($avg > $sdev)
          $mdev = $avg - $sdev;
        else
          $mdev = $sdev - $avg;

        //number_format(round($ms, 3), 3, '.', '')

        echo 	''
		."\n".'--- '.$this->host.' ping ['.$this->name.'] statistics ---'."\n"
		.$this->icmp_seq.' packets transmitted, XX received, XX% packetloss, '
		.'time '.(time() - $this->time_start).'s'
		."\n"
		.'rtt min/avg/max/mdev = '
		.number_format(round($this->min, 3), 3, '.', '').'/'
		.number_format(round($avg, 3), 3, '.', '').'/'
		.number_format(round($this->max, 3), 3, '.', '').'/'
		.number_format(round($mdev, 3), 3, '.', '')
		.' ms'
		."\n\n";

        if (($this->pid == 0 && $this->pid_child == 0) OR ($this->pid == $this->pid_child)) //rotate mode
        {
          sonic_daemon_thread_shutdown($sig);
          $this->rotate_shutdown = true;
        }
        else
        {
          $this->sonic_stream_ping_stop();
          $this->sonic_stream_buffer();
          exit();
        }
      break;

      default:
        die("ping says WTF");
    }
  }

  /**
   * main setup
   *
   * init time start, reset internal variables and
   * load signal handers and setup socket out connectiong then return
   *
   * @see SONIC_STREAM_ping::sonic_stream_main()
  */
  function sonic_stream_ping_start()
  {
    if ($this->pcntl_loaded)
    { 
      pcntl_signal(SIGTERM, array(&$this, "_sonic_stream_ping_handler"));
      pcntl_signal(SIGQUIT, array(&$this, "_sonic_stream_ping_handler"));
      pcntl_signal(SIGINT, array(&$this, "_sonic_stream_ping_handler"));
    }

//    var_dump($this->config);

    if (($_bs = 0 + $this->config["byte_size"]) > 0) {}
    else { $_bs = 65535; }
    define("SONIC_PING_BYTE_SIZE", $_bs); unset($_bs);

    $stream_config = &$this->config["stream_config"];

    if ($this->verbose > 0)
    {
      if ($msg = $stream_config["example_msg1"])
        $msg .= "\n";
      else
        $msg = "creating connection until close\n";

      echo '= ['.posix_getpid().'] '
          .date(SONIC_DATEF, time())
          .' ['.$this->name.'] '
          .'';
      echo $msg;
    }

    if ($this->host = $stream_config["host"]);
    else
      die("Error: no destination host given");

    $this->timeout_max = 300 * 1000; //ms

    $this->port = 0;
 
    //xxbit - but less than MTU eg. 1byte=8bit=1chr()
    $this->data_size = 7; //56 bytes (7 * 8) + 8byte header = 64 bytes = 512 bits packet size

    $this->socket = socket_create(AF_INET, SOCK_RAW, getprotobyname('icmp'));

    $this->mdev_total = 0;
    $this->icmp_seq = 0;

    $this->min = $this->avg = $this->max = $this->mdev = 0;

    $this->time_start = time();
  }

  /**
   * main shutdown/finish
   *
   * closes out resource socket variable and echos trace
   *
   * @see SONIC_STREAM_ping::sonic_stream_main()
  */
  function sonic_stream_ping_stop()
  {
    $stream_config = $this->config["stream_config"];

    socket_close($this->socket);

    if ($this->verbose > 0)
    {
      if ($msg_close = $stream_config["example_msg2"])
        $msg_close .= "\n";
      else
        $msg_close = "connection finished\n";

      echo '= ['.posix_getpid().'] '
          .date(SONIC_DATEF, time())
          .' ['.$this->name.'] '
          .'';
      echo $msg_close;
    }
  }

  /**
   * stream main
   *
   * @see SONIC_stream::sonic_stream_main()
  */
  function sonic_stream_main()
  {
    $this->sonic_stream_ping_start();

    $stream_config = $this->config["stream_config"];

    $host = &$this->host;
    $port = &$this->port;
    $icmp_seq = &$this->is;
    $timeout_max = &$this->timeout_max;
    $data_size = &$this->data_size;
    $ms_total = &$this->ms_total;
    $mdev_total = &$this->mdev_total;
    $min = &$this->min;
    $avg = &$this->avg;
    $max = &$this->max;
    $mdev = &$this->mdev;

    $sock = &$this->socket;

    $out_ok = 'OK';
    $out = '';

    $ident0 = rand(0, 255);
    $ident1 = rand(0, 255);
    $ident = array($ident0, $ident1);

    $seq0 = rand(0, 255);
    $seq1 = rand(0, 255);
    $seq = array($seq0, $seq1);

    $is = &$this->icmp_seq;
    $is_count = 9; //0;
//    $is_count = 0;


    //inital microtime pre-cache hack?
    //nfi php optimization getting in the way
    //of ticked/true times, some inital cache
    //is happening somewhere - remove this
    //and watch the inital ping time
//    usleep(5);
//    microtime(); microtime();
//    usleep(5);

    $ignore_inital_hack = false;

    do
    {
      $this->sonic_stream_buffer();

      $out = $out_ok;

      $is++;

      if (($is_count > 0) && ($is > $is_count))
        break;

//TODO maybe func init this, keep in loop?, arlo?
      //http://en.wikipedia.org/wiki/Ping#Echo_request
      $packet = ''
	.chr(8)		// type = 8	// 8bit	//1,
	.chr(0)		// code = 0	// 8bit	//2,
	.chr(0).chr(0)	// checksum	//16bit	//AND a..
			//ident
        .chr($ident[0]).chr($ident[1])	//16bit	//5, 6
			//seq
        .chr($seq[0]).chr($seq[1])	//16bit	//7, 8
	.'';

      for ($i = 0; $i < $data_size; $i++)
        $packet .= chr(0);

      $cksum = sonic_checksum($packet, 16);

      $packet[2] = $cksum[0]; // checksum init	//3,
      $packet[3] = $cksum[1]; // checksum init	//4
//end maybe

      list($time_start, $time_startu) = preg_split('/\s/', microtime());

      $this->idle = false;

      socket_sendto($sock, $packet, strlen($packet), 0, $host, $port);

      $read = array($sock);
      $write  = NULL;
      $except = NULL;
      $select = socket_select($read, $write, $except, 0, $timeout_max);

      list($time_stop, $time_stopu) = preg_split('/\s/', microtime());

      $ms = (($time_stopu + (float) $time_stop) - ($time_startu + (float) $time_start)) * 1000;

      //WARN, will have to clear buffer even if timeout
      //WARN, host n port can change, its pass by ref
      $recv = '';
      $rhost = $host;
      $rport = $port;
      socket_recvfrom($sock, $recv, SONIC_PING_BYTE_SIZE, 0, $rhost, $rport);

      if ($ignore_inital_hack);
      else
      {
        $ignore_inital_hack = true;
        $is--;
        continue;
      }

      $this->idle = true;

      if ($rhost == $host);
      else
      {
        echo '= ['.posix_getpid().'] '
          .date(SONIC_DATEF, time())
          .' ['.$this->name.'] '
          .'';

        echo "Ummm.. thong thong thong, $rhost -> $host\n";
        break; //TODO, WTF
      }

      $recv = unpack('C*', $recv);
//echo str_replace("\n", " ", var_export($recv, true));
//echo "\n";

      if ($select == 0)
      {
        $out = "Timeout";
      }
      else if ($recv[10] != 1)
      {
        $out = "Not ICMP packet";
      }
      else if ($recv[21] != 0 AND $recv[21] != 8) //8 for local reply? nfi TODO verify
      {
        $out = "Not ICMP response";
      }
      else if ($ident[0] != $recv[25] OR $ident[1] != $recv[26])
      {
        $out = "Bad identification number";
      }
      else if ($seq[0] != $recv[27] OR $seq[1] != $recv[28])
      {
        $out = "Bad sequence number";
      }
      else if ($ms < 0)
      {
        $out = "Response too long";
      }

      if ($ms > 0)
      {
        $ms_total += $ms;
        $mdev_total += $ms * $ms;

        if ($this->min > $ms OR $this->min == 0)
           $this->min = $ms;

        if ($this->max < $ms OR $this->max == 0)
           $this->max = $ms;
      }

      if ($this->verbose > 0)
      {
        echo '= ['.posix_getpid().'] '
          .date(SONIC_DATEF, time())
          .' ['.$this->name.'] '
          .'';

        echo ''	.(($data_size * 8) + 8).' bytes from '.$host
		.': icmp_seq='.$is
//		.' ttl=NA'
		.' time='
		.($ms < 0 ? $ms : number_format(round($ms, 3), 3, '.', ''))
		.' '.$out
		."\n";
      }

      sleep(1);

      if ($this->rotate_shutdown) //roate mode
        break;

    } while (true);

    if ($this->verbose > 0)
    {
      echo '= ['.posix_getpid().'] '
          .date(SONIC_DATEF, time())
          .' ['.$this->name.'] '
          .'';
      echo $out."\n";
    }

    $this->sonic_stream_ping_stop();
  }//main()
}//class

?>
