<?
/* --------------------------------------------------------------------
   Copyright (c) 2008-2012 Daniel Pedemont (dan@pedemont.net)
   -------------------------------------------------------------------- */
/**
 * stream remote
 *
 * stream example 101 (sockets and signals)
 *
 * @link http://dev.pedemont.com/sonic/
 * @link http://dev.pedemont.com/sonic/downloads/src/lib/streams/sonic.stream.remote.php
 * @package sonic_stream
 * @version 0.2.12
 * @abstract remote daemon example
*/

define('SONIC_REMOTE_VERSION', '0.2.12');

//NB, function names in rotate mode have no protection,
//    so dont use generic function names for clash safety

class SONIC_STREAM_remote extends SONIC_stream
{
  /**
   * signal termination handler
   *
   * @param integer $sig posix process signal
  */
  function _sonic_stream_remote_handler($sig = NULL)
  {
//echo "_sonic_stream_remote_handler()\n";
    //none of this is really needed, PHP will clean up everything
    //but useful for error messages with server reset in IPC mode
    //
    //in thread mode, sonic_thread_handler() will close
    //socket on last instance/thread anyway
    switch ($sig)
    { 
      case SIGTERM:
      case SIGQUIT:
      case SIGINT:
        if (($this->pid == 0 && $this->pid_child == 0) OR ($this->pid == $this->pid_child)) //rotate mode
        { 
          sonic_daemon_thread_shutdown($sig);
          $this->rotate_shutdown = true;
        }
        else
        { 
          $this->sonic_stream_remote_stop();
          $this->sonic_stream_buffer();
          exit();
        }
      break;

      default:
        die("remote says WTF");
    }
  }

  /**
   * main setup
   *
   * load signal handers and
   * poll socket_accept listening socket for connected client then return
   *
   * @see SONIC_STREAM_remote::sonic_stream_main()
   * @see SONIC_STREAM_remote::sonic_stream_remote_prelog()
   * @param resource third party support (library mode) override, listening resource socket
  */
  function sonic_stream_remote_start($sock_in = NULL)
  {
    if ($this->pcntl_loaded)
    {
      pcntl_signal(SIGTERM, array(&$this, "_sonic_stream_remote_handler"));
      pcntl_signal(SIGQUIT, array(&$this, "_sonic_stream_remote_handler"));
      pcntl_signal(SIGINT, array(&$this, "_sonic_stream_remote_handler"));
    }

    if ($sock_in)			//3rd party compat
      $this->sock = &$sock_in;
    else if ($this->sock)		//sonic daemon inherit preload
    {
      $sock_in = &$this->sock;
    }
    else if ($this->config["socket"])	//despro search
    {
      $sock_in = &$this->config["socket"];
    }
    else
      die('! '.$this->sonic_stream_remote_prelog().' main() failed: reason: no socket available'."\n");

//    var_dump($this->config);

    if (($_bs = 0 + $this->config["byte_size"]) > 0) {}
    else { $_bs = 2048; }
    define("SONIC_REMOTE_BYTE_SIZE", $_bs); unset($_bs);

    $stream_config = $this->config["stream_config"];

    if ($this->verbose > 0)
    {
      if ($msg = $stream_config["example_msg1"])
        $msg .= "\n";
      else
        $msg = "listening for connection until close\n";

      echo '= '.$this->sonic_stream_remote_prelog().' ';
      echo $msg;
    }

    if ($this->config['socket_type'] == SOCK_STREAM)
    {
      $this->sonic_stream_buffer();

      if ($_sock_in = $this->sonic_stream_socket_accept($sssa_msg))
      {
        if ($this->verbose > 0)
          echo '= '.$this->sonic_stream_remote_prelog().' '.$sssa_msg."\n";
      }
      else
        die('! '.$this->sonic_stream_remote_prelog().' main() failed: reason: '.$sssa_msg."\n");

      //WARN, overwrite root with accept socket
      $this->sock = $_sock_in;
      unset($_sock_in);
      $sock_in = &$this->sock;
      
      if ($stream_config["tcp_echo"])
        $stream_config["tcp_echo"] = true;
      else
        $stream_config["tcp_echo"] = false;
    }
    else
      $stream_config["tcp_echo"] = false;
  }

  /**
   * main shutdown/finish
   *
   * closes resource socket in variable and echos trace
   *
   * @see SONIC_STREAM_remote::sonic_stream_main()
   * @see SONIC_STREAM_remote::sonic_stream_remote_prelog()
  */
  function sonic_stream_remote_stop()
  {
    if ($this->verbose > 0)
    {
      $stream_config = $this->config["stream_config"];

      if ($msg_close = $stream_config["example_msg2"])
        $msg_close .= "\n";
      else
        $msg_close = "connection finished\n";

      echo '= '.$this->sonic_stream_remote_prelog().' ';
      echo $msg_close;
    }

    if ($this->sock);
      socket_close($this->sock);
  }

  /**
   * stream main
   *
   * @see SONIC_stream::sonic_stream_main()
   * @param resource $sock_in third party support (library mode) override, resource socket bound and listening for connections
  */
  function sonic_stream_main($sock_in = NULL)
  {
    $this->sonic_stream_remote_start($sock_in);

    $sock_in = &$this->sock;

    //quit after accept
//    if ((($this->pid == 0 && $this->pid_child == 0) OR ($this->pid == $this->pid_child)) && $this->rotate_shutdown)
    if ($this->rotate_shutdown)
    { 
      $this->sonic_stream_remote_stop();
      return;
    }

    $stream_config = $this->config["stream_config"];

    if ($this->verbose == 1)
    {
      echo '= '.$this->sonic_stream_remote_prelog().' ';
      echo 'NB, use verbose set level 2 for base64_encode() or set higher for raw output'."\n";
    }

    $this->idle = false;

    do
    {
      $this->sonic_stream_buffer();

      if (false == ($buf = @socket_read($sock_in, SONIC_REMOTE_BYTE_SIZE, PHP_BINARY_READ)))
      {
        break;
      }
      else if ($this->verbose == 1)
      {
        echo '= '.$this->sonic_stream_remote_prelog().' ';
        echo 'strlen($buffer) = '.strlen($buf)."\n";
      }
      else if ($this->verbose == 2)
      {
        echo base64_encode($buf);
      }
      else if ($this->verbose > 0)
      {
        echo $buf;
      }

      if ($stream_config["tcp_echo"] && $this->config["socket_type"] == SOCK_STREAM)
        socket_write($sock_in, $buf, strlen($buf));

//      if ((($this->pid == 0 && $this->pid_child == 0) OR ($this->pid == $this->pid_child)) && $this->rotate_shutdown)
      if ($this->rotate_shutdown)
        break;

    } while (true);

    $this->idle = true;

    $this->sonic_stream_remote_stop();
  }

  /**
   * log output unifier
   *
   * returns a uniform prefix for log/output entries
   *
   * @see SONIC_stream::sonic_stream_prelog()
   * @return string string usually with pid/process id and timestamp
  */
  function sonic_stream_remote_prelog()
  {
    return $this->sonic_stream_prelog();
  }
}

?>
